// The Nature of Code
// Daniel Shiffman
// http://natureofcode.com

// Modified by Matt Borland for UWTMC to make music by sending out MIDI messages

// Press keys in the "A" to ";" row of your keyboard to create particles 
// where your mouse cursor is located.
// Each particle has a note associated with its radius and will send a
// MIDI message when it contacts a boundary or another particle

// Install the following libraries by going to the menu bar -> Sketch/Import Library/Add Library/
// Then search for and install:
// "the MidiBus"
// "Box2D for Processing"

import themidibus.*; //Import the library "The Midibus" 
MidiBus myBus; // Create a MIDI bus instance to send and receive MIDI messages
int channel = 0;    // Zero-referenced MIDD channel to send messages on (values from 0-15)

// There are 10 keys you an press, so you need 10 pitch values here or else you'll create an error
// message and the program will crash
// These use the standard MIDI note numbers
int[] pitch = {60, 62, 64, 67, 69, 72, 74, 76, 79, 81}; // 

// Some variables to set the properties of the physics simulation
int smallRadius = 10; // the smallest particle size
int bigRadius = 100;   // the largest particle size
int GravityValue = -100; // the value of the gravity in the world

// Import a bunch of libraries associated with the Box2D for Processing library
// These libraries provide the 2D physics code we'll use to simulate gravity
import shiffman.box2d.*;
import org.jbox2d.common.*;
import org.jbox2d.dynamics.joints.*;
import org.jbox2d.collision.shapes.*;
import org.jbox2d.collision.shapes.Shape;
import org.jbox2d.common.*;
import org.jbox2d.dynamics.*;
import org.jbox2d.dynamics.contacts.*;

// A reference to our box2d world
Box2DProcessing box2d;

// An ArrayList of particles that will fall on the surface
ArrayList<Particle> particles;

// Boundary Variables to be create virtual walls the particels will bounce off of
Boundary wall1;
Boundary wall2;

void setup() {
  fullScreen(); // We're going to run this as a fullscreen program - press "esc" to escape
  
  // Initialize box2d physics and create the world
  box2d = new Box2DProcessing(this);
  box2d.createWorld();
  box2d.setGravity(0, GravityValue);

  // Turn on collision listening!
  box2d.listenForCollisions();

  // Create the empty list
  particles = new ArrayList<Particle>();

  // Create the Boundary elements
  wall1 = new Boundary(width/2, height-5, width, 10);
  wall2 = new Boundary(width/2, -5, width, 10);

  // List the available MIDI ports
  MidiBus.list(); // List all available 

  // Connect the MidiBus to an available port - 
  // check what's available in the "console" in this window below the text editor
  //myBus = new MidiBus(this, "vMIDI", "vMIDI"); // Create a new MidiBus
  myBus = new MidiBus(this, "IAC Bus 1", "IAC Bus 1"); // Create a new MidiBus
}

void draw() {
  // Redraw the background white each time (this has the effect of clearing the screen)
  background(10); // try commenting this out to leave trails behind the particles 

  // We must always step through time! tell the simulation to calculate the next time step.
  box2d.step();

  // Look at all particles
  for (int i = particles.size()-1; i >= 0; i--) {
    Particle p = particles.get(i);
    p.display();
    
    // Particles that leave the screen, we delete them
    // note that they have to be deleted from both the box2d world and our list
    if (p.done()) {
      particles.remove(i);
    }
  }
  // Draw the wall boundaries defined earlier
  wall1.display();
  wall2.display();
}

// Collision event functions! Check when two objects collide and figure out how to respond
void beginContact(Contact cp) {

  // Get both fixtures
  Fixture f1 = cp.getFixtureA();
  Fixture f2 = cp.getFixtureB();

  // Get both bodies frmo the fixtures
  Body b1 = f1.getBody();
  Body b2 = f2.getBody();

  // Get our objects that reference these bodies
  Object o1 = b1.getUserData();
  Object o2 = b2.getUserData();

  // If the colliding objects are both particles, then play both notes
  if (o1.getClass() == Particle.class && o2.getClass() == Particle.class) {

    Particle p1 = (Particle) o1;
    p1.change();
    int n1 = int(map(p1.radius(), bigRadius, smallRadius, 0, 9));
    myBus.sendNoteOn(channel, pitch[n1], 100); // Send a Midi noteOn
    
    Particle p2 = (Particle) o2;
    p2.change();
    int n2 = int(map(p2.radius(), bigRadius, smallRadius, 0, 9));
    myBus.sendNoteOn(channel, pitch[n2], 100); // Send a Midi noteOn

    delay(10);
    myBus.sendNoteOff(channel, pitch[n1],0); // Send a Midi noteOff
    myBus.sendNoteOff(channel, pitch[n2],0); // Send a Midi noteOff
  
  // If particle 1 hits a wall, just play one note
  } else if (o1.getClass() == Particle.class) {
    Particle p1 = (Particle) o1;
    p1.change();
    int n1 = int(map(p1.radius(), bigRadius, smallRadius, 0, 9));
    myBus.sendNoteOn(channel, pitch[n1], 100); // Send a Midi noteOn
    delay(10);
    myBus.sendNoteOff(channel, pitch[n1],0); // Send a Midi noteOff
  
  // If particle 2 hits a wall, just play one note  
  } else if (o2.getClass() == Particle.class) {
    Particle p2 = (Particle) o2;
    p2.change();
    int n2 = int(map(p2.radius(), bigRadius, smallRadius, 0, 9));
    myBus.sendNoteOn(channel, pitch[n2], 100); // Send a Midi noteOn
    delay(10);
    myBus.sendNoteOff(channel, pitch[n2],0); // Send a Midi noteOff
  }  
}

// Objects stop touching each other
void endContact(Contact cp) {
}

// This code will create particles of different sizes depending on where your mouse currently is
// when you press a key in the a to ; row of your keyboard
void keyPressed(){
  if(key == 'a'){
    int sz = int(map(0, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));
  } else if (key == 's'){
    int sz = int(map(1, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));  
  } else if (key == 'd'){
    int sz = int(map(2, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));    
  } else if (key == 'f'){
    int sz = int(map(3, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));    
  } else if (key == 'g'){
    int sz = int(map(4, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));    
  } else if (key == 'h'){
    int sz = int(map(5, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));    
  } else if (key == 'j'){
    int sz = int(map(6, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));    
  } else if (key == 'k'){
    int sz = int(map(7, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));   
  } else if (key == 'l'){
    int sz = int(map(8, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));    
  } else if (key == ';'){  
     int sz = int(map(9, 0, 9, bigRadius, smallRadius)); 
    particles.add(new Particle(mouseX, mouseY, sz));
  }
}
